package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.utility.BookHelper;
import net.knarcraft.bookswithoutborders.utility.EconomyHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import net.knarcraft.bookswithoutborders.utility.TabCompletionHelper;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * Command executor for the copy command
 */
public class CommandCopy implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return false;
        }

        if (InventoryHelper.notHoldingOneWrittenBookCheck(player, "You must be holding a written book to copy it!",
                "You cannot copy two books at once!")) {
            return false;
        }

        if (args.length < 1) {
            BooksWithoutBorders.sendErrorMessage(player, "You must specify the number of copies to be made!");
            return false;
        }

        //TODO: Rewrite this so the resulting book becomes a COPY, or COPY OF COPY


        try {
            ItemStack heldBook = InventoryHelper.getHeldBook(player, true);
            int copies = Integer.parseInt(args[0]);
            if (copies <= 0) {
                throw new NumberFormatException("Number of copies must be larger than 0");
            }
            return performCopy(copies, player, heldBook);
        } catch (NumberFormatException ignored) {
            BooksWithoutBorders.sendErrorMessage(player, "Book not copied!");
            BooksWithoutBorders.sendErrorMessage(player, "Number specified was invalid!");
            return false;
        }
    }

    /**
     * Performs the actual copying
     *
     * @param copies   <p>The number of copies to be made</p>
     * @param player   <p>The player requesting the copies</p>
     * @param heldBook <p>The book to be copied</p>
     * @return <p>True if the copying was successful</p>
     */
    private boolean performCopy(int copies, Player player, ItemStack heldBook) {
        //Make sure the player owns the book if authorOnlyCopy is enabled
        if (BooksWithoutBordersConfig.getAuthorOnlyCopy() &&
                !player.hasPermission("bookswithoutborders.bypassAuthorOnlyCopy")) {
            if (BookHelper.isNotAuthor(player, (BookMeta) Objects.requireNonNull(heldBook.getItemMeta()))) {
                return false;
            }
        }

        //Make sure the player can pay for the copying
        if (BooksWithoutBordersConfig.booksHavePrice() &&
                !player.hasPermission("bookswithoutborders.bypassBookPrice") &&
                EconomyHelper.cannotPayForBookPrinting(player, copies)) {
            return false;
        }

        BookMeta bookMeta = (BookMeta) heldBook.getItemMeta();
        if (BooksWithoutBordersConfig.changeGenerationOnCopy() && bookMeta != null && bookMeta.hasGeneration()) {
            return copyNextGenerationBook(bookMeta, player, copies);
        } else {
            //Make sure the player can pay for the copying
            if (paymentUnSuccessful(player, copies)) {
                return false;
            }

            heldBook.setAmount(heldBook.getAmount() + copies);
            BooksWithoutBorders.sendSuccessMessage(player, "Book copied!");
            return true;
        }
    }

    /**
     * Tries to take payment from a player
     *
     * @param player <p>The player to take the payment from</p>
     * @param copies <p>The number of copies to create for the player</p>
     * @return <p>True if the payment failed</p>
     */
    private boolean paymentUnSuccessful(Player player, int copies) {
        return BooksWithoutBordersConfig.booksHavePrice() &&
                !player.hasPermission("bookswithoutborders.bypassBookPrice") &&
                EconomyHelper.cannotPayForBookPrinting(player, copies);
    }

    /**
     * Copies a book with the next generation relative to the input book
     *
     * @param bookMeta <p>The book to copy</p>
     * @param player   <p>The player copying the book</p>
     * @param copies   <p>The number of copies requested</p>
     * @return <p>True if the book was successfully copied</p>
     */
    private boolean copyNextGenerationBook(BookMeta bookMeta, Player player, int copies) {
        //Copy the vanilla behavior of refusing copying any further
        if (bookMeta.getGeneration() == BookMeta.Generation.COPY_OF_COPY) {
            player.sendMessage("You cannot copy this book any further. You must have the original or a direct copy.");
            return false;
        }
        //Make sure the player can fit the book in their inventory
        int nextAvailableSlot = player.getInventory().firstEmpty();
        if (nextAvailableSlot == -1) {
            player.sendMessage("You need an available slot in your inventory.");
            return false;
        }
        //Make sure the player can pay for the copying
        if (paymentUnSuccessful(player, copies)) {
            return false;
        }

        ItemStack itemStack = new ItemStack(Material.WRITTEN_BOOK);
        itemStack.setItemMeta(bookMeta);
        //Increase the generation of the book
        BookHelper.increaseGeneration(itemStack);
        itemStack.setAmount(copies);

        player.getInventory().addItem(itemStack);
        return true;
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias,
                                      @NotNull String[] args) {
        int argumentCount = args.length;
        if (argumentCount == 1) {
            return TabCompletionHelper.filterMatchingStartsWith(TabCompletionHelper.getNumbers(1, 20), args[0]);
        }
        return new ArrayList<>();
    }

}
