package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.utility.BookLoader;
import net.knarcraft.bookswithoutborders.utility.FileHelper;
import net.knarcraft.bookswithoutborders.utility.InputCleaningHelper;
import net.knarcraft.bookswithoutborders.utility.TabCompletionHelper;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * Command executor for the load command
 */
public class CommandLoad implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        return loadBook(sender, args, "player", false);
    }

    /**
     * Loads a stored book
     *
     * @param sender     <p>The sender asking to load the book</p>
     * @param args       <p>The arguments given</p>
     * @param directory  <p>The directory to load from (public/player)</p>
     * @param loadPublic <p>Whether to list public files as loadable</p>
     * @return <p>True if the book was loaded successfully</p>
     */
    boolean loadBook(CommandSender sender, String[] args, String directory, boolean loadPublic) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return false;
        }

        if (player.getInventory().firstEmpty() == -1) {
            BooksWithoutBorders.sendErrorMessage(player, "You must have space in your inventory to load books!");
            return false;
        }

        int argumentCount = args.length;

        //Show books available to the player
        if (argumentCount == 0) {
            FileHelper.printBooks(sender, loadPublic);
            return true;
        }

        //Organize and parse input
        String bookIdentifier = args[0];
        String copies = "1";
        String isSigned = "true";
        if (args.length == 3) {
            copies = args[1];
            isSigned = args[2];
        } else if (args.length == 2) {
            if (args[1].equalsIgnoreCase("true") || args[1].equalsIgnoreCase("false")) {
                isSigned = args[1];
            } else {
                copies = args[1];
            }
        }

        //Load books available to the player
        try {
            Integer.parseInt(bookIdentifier);
            BooksWithoutBorders.updateBooks(sender, loadPublic);
        } catch (NumberFormatException ignored) {
        }

        String bookToLoad = InputCleaningHelper.cleanString(bookIdentifier);
        try {
            //Give the new book if it can be loaded
            ItemStack newBook = BookLoader.loadBook(player, bookToLoad, isSigned, directory, Integer.parseInt(copies));
            if (newBook != null) {
                player.getInventory().addItem(newBook);
                BooksWithoutBorders.sendSuccessMessage(player, "Book created!");
                return true;
            } else {
                BooksWithoutBorders.sendErrorMessage(player, "Book failed to load!");
                return false;
            }
        } catch (NumberFormatException e) {
            BooksWithoutBorders.sendErrorMessage(player, "Invalid number of book copies specified!");
            return false;
        }
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias, @NotNull String[] args) {
        return doTabCompletion(sender, args, false);
    }

    /**
     * Performs the actual tab completion
     *
     * @param sender     <p>The sender of the command</p>
     * @param args       <p>The arguments given</p>
     * @param loadPublic <p>Whether to list public files or player files</p>
     * @return <p>A list of available choices</p>
     */
    protected List<String> doTabCompletion(CommandSender sender, String[] args, boolean loadPublic) {
        int argumentCount = args.length;
        if (argumentCount == 1) {
            //Return list of books
            return TabCompletionHelper.filterMatchingContains(BooksWithoutBorders.getAvailableBooks(sender, loadPublic),
                    args[0]);
        } else if (argumentCount == 2) {
            //Number of copies
            return TabCompletionHelper.filterMatchingStartsWith(TabCompletionHelper.getBooleansAndNumbers(1, 3), args[1]);
        } else if (argumentCount == 3) {
            //Signed
            try {
                Integer.parseInt(args[1]);
                return TabCompletionHelper.filterMatchingStartsWith(TabCompletionHelper.getBooleans(), args[2]);
            } catch (NumberFormatException e) {
                return new ArrayList<>();
            }
        }
        return new ArrayList<>();
    }

}
