package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.state.ItemSlot;
import net.knarcraft.bookswithoutborders.utility.BookHelper;
import net.knarcraft.bookswithoutborders.utility.BookToFromTextHelper;
import net.knarcraft.bookswithoutborders.utility.FileHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getCommandColor;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getErrorColor;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getTitleAuthorSeparator;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.cleanString;

/**
 * Command executor for the save command
 */
public class CommandSave implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, String[] args) {
        return saveHeldBook(sender, args, false);
    }

    /**
     * Saves the player's held book if it exists
     *
     * @param sender     <p>The sender of the command</p>
     * @param args       <p>The arguments given</p>
     * @param savePublic <p>Whether to save the book in the public directory or the player directory</p>
     * @return <p>True if a book was saved successfully</p>
     */
    boolean saveHeldBook(CommandSender sender, String[] args, boolean savePublic) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return false;
        }

        ItemSlot holdingSlot = InventoryHelper.getHeldSlotBook(player, false, false, false, false);
        if (holdingSlot != null && holdingSlot != ItemSlot.NONE) {
            ItemStack holdingItem = InventoryHelper.getHeldItem(player, holdingSlot == ItemSlot.MAIN_HAND);
            boolean duplicate = args.length == 1 && Boolean.parseBoolean(args[0]);
            saveBook(player, holdingItem, duplicate, savePublic);
            return true;
        } else {
            BooksWithoutBorders.sendErrorMessage(sender, "You must be holding a written book or book and quill to save it!");
            return false;
        }
    }

    /**
     * Saves a book to a file
     *
     * @param player             <p>The player holding the book</p>
     * @param heldBook           <p>The book held</p>
     * @param overwrite          <p>Whether to overwrite any existing books</p>
     * @param saveToPublicFolder <p>Whether to save the book to the public folder instead of the player folder</p>
     */
    public void saveBook(Player player, ItemStack heldBook, boolean overwrite, boolean saveToPublicFolder) {
        BookMeta book = (BookMeta) heldBook.getItemMeta();
        if (book == null) {
            BooksWithoutBorders.sendErrorMessage(player, "Unable to get metadata for your held book!");
            return;
        }

        String savePath;
        if (saveToPublicFolder) {
            savePath = getBookFolder();
        } else {
            savePath = getBookFolder() + cleanString(player.getName()) + getSlash();
        }

        //Generate book filename
        String fileName = BookHelper.getBookFile(book, player);

        //Make sure the used folders exist
        File file = new File(savePath);
        if (!file.exists() && !file.mkdir()) {
            BooksWithoutBorders.sendErrorMessage(player, "Saving Failed! If this continues to happen, consult server admin!");
            return;
        }
        File[] foundFiles = file.listFiles();
        if (foundFiles == null) {
            BooksWithoutBorders.sendErrorMessage(player, "Saving Failed! If this continues to happen, consult server admin!");
            return;
        }

        //Find any duplicates of the book
        int foundDuplicates = FileHelper.findDuplicates(foundFiles, fileName);

        //Deal with duplicates
        if (foundDuplicates > 0) {
            //TODO: Decide if this makes sense or needs to be changed
            //Skip duplicate book
            if (!fileName.contains("Untitled" + getTitleAuthorSeparator()) && !overwrite) {
                BooksWithoutBorders.sendErrorMessage(player, "Book is already saved!");
                BooksWithoutBorders.sendErrorMessage(player, "Use " + getCommandColor() + "/savebook true " +
                        getErrorColor() + "to overwrite!");
                return;
            }

            //Skip if duplicate limit is reached
            if (foundDuplicates > BooksWithoutBordersConfig.getBookDuplicateLimit()) {
                BooksWithoutBorders.sendErrorMessage(player, "Maximum amount of " + fileName +
                        " duplicates reached!");
                BooksWithoutBorders.sendErrorMessage(player, "Use " + getCommandColor() + "/savebook true " +
                        getErrorColor() + "to overwrite!");
                return;
            }

            //Alter duplicated filename
            if (fileName.contains("Untitled" + getTitleAuthorSeparator()) && !overwrite) {
                fileName = "(" + foundDuplicates + ")" + fileName;
            }
        }

        try {
            if (BooksWithoutBordersConfig.getUseYml()) {
                BookToFromTextHelper.bookToYml(savePath, fileName, book);
            } else {
                BookToFromTextHelper.bookToTXT(savePath, fileName, book);
            }

            //Update the relevant book list
            BooksWithoutBorders.updateBooks(player, saveToPublicFolder);
            BooksWithoutBorders.sendSuccessMessage(player, "Book Saved as \"" + fileName + "\"");
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias,
                                      @NotNull String[] args) {
        return new ArrayList<>();
    }

}
