package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.utility.EconomyHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import net.knarcraft.bookswithoutborders.utility.TabCompletionHelper;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * Command executor for the set book price command
 */
public class CommandSetBookPrice implements TabExecutor {

    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();
    private List<String> paymentTypes;

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, String[] args) {
        //Clear the current price
        if (args.length == 0) {
            clearItemPrice(sender);
            return true;
        }

        //Warn about missing arguments
        if (args.length < 2) {
            BooksWithoutBorders.sendErrorMessage(sender, "[Item/Eco] and [quantity] must be specified!");
            return false;
        }

        //Warn about invalid argument
        if (!args[0].equalsIgnoreCase("Item") && !args[0].equalsIgnoreCase("Eco")) {
            BooksWithoutBorders.sendErrorMessage(sender, "Price type must be \"Item\" or \"Eco\"!");
            return false;
        }

        try {
            double price = Double.parseDouble(args[1]);
            if (price <= 0) {
                BooksWithoutBorders.sendErrorMessage(sender, "[quantity] must be greater than 0!");
                return false;
            }

            if (args[0].equalsIgnoreCase("Item")) {
                return setItemPrice(sender, price);
            } else if (args[0].equalsIgnoreCase("Eco")) {
                return setEconomyPrice(sender, price);
            }
        } catch (NumberFormatException e) {
            BooksWithoutBorders.sendErrorMessage(sender, "[quantity] must be a number!");
        }
        return false;
    }

    /**
     * Removes the book price
     *
     * @param sender <p>The sender of the command</p>
     */
    private void clearItemPrice(CommandSender sender) {
        BooksWithoutBordersConfig.setBookPriceType(null);
        BooksWithoutBordersConfig.setBookPriceQuantity(0);
        booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Item_type", "Item type name");
        booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Required_quantity", BooksWithoutBordersConfig.getBookPriceQuantity());
        booksWithoutBorders.saveConfig();

        BooksWithoutBorders.sendSuccessMessage(sender, "Price to create books removed!");
    }

    /**
     * Sets the book price to use items, and updates book price
     *
     * @param sender <p>The sender of the command</p>
     * @param price  <p>The new price</p>
     * @return <p>True if the price was changed successfully</p>
     */
    private boolean setItemPrice(CommandSender sender, double price) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "[Item] price can only be used by a player!");
            return false;
        }

        ItemStack heldItem = InventoryHelper.getHeldItem(player, true);
        if (heldItem.getType() == Material.AIR) {
            BooksWithoutBorders.sendErrorMessage(sender, "Must be holding an item to set an [Item] price!");
            return false;
        }

        BooksWithoutBordersConfig.setBookPriceType(heldItem.getType());
        BooksWithoutBordersConfig.setBookPriceQuantity(price);
        String newPriceType = BooksWithoutBordersConfig.getBookPriceType().toString();
        double newPriceQuantity = BooksWithoutBordersConfig.getBookPriceQuantity();
        booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Item_type", newPriceType);
        booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Required_quantity", newPriceQuantity);
        booksWithoutBorders.saveConfig();

        BooksWithoutBorders.sendSuccessMessage(sender, "Book creation price set to " + (int) newPriceQuantity +
                " " + newPriceType + "(s)!");
        return true;
    }

    /**
     * Sets the book price to use economy, and updates book price
     *
     * @param sender <p>The sender of the command</p>
     * @param price  <p>The new price</p>
     * @return <p>True if the price was changed successfully</p>
     */
    private boolean setEconomyPrice(CommandSender sender, double price) {
        if (EconomyHelper.setupEconomy()) {
            BooksWithoutBordersConfig.setBookPriceQuantity(price);
            BooksWithoutBordersConfig.setBookPriceType(Material.AIR);
            double newPriceQuantity = BooksWithoutBordersConfig.getBookPriceQuantity();
            booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Item_type", "Economy");
            booksWithoutBorders.getConfig().set("Options.Price_to_create_book.Required_quantity", newPriceQuantity);
            booksWithoutBorders.saveConfig();

            BooksWithoutBorders.sendSuccessMessage(sender, "Book creation price set to " +
                    EconomyHelper.getEconomy().format(newPriceQuantity) + "!");
            return true;
        } else {
            BooksWithoutBorders.sendErrorMessage(sender, "BooksWithoutBorders failed to hook into Vault! Book price not set!");
            return false;
        }
    }

    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String alias, String[] args) {
        if (paymentTypes == null) {
            initializeTabCompleteLists();
        }

        int argumentCount = args.length;
        if (argumentCount == 1) {
            return TabCompletionHelper.filterMatchingStartsWith(paymentTypes, args[0]);
        } else if (argumentCount == 2) {
            return TabCompletionHelper.filterMatchingStartsWith(TabCompletionHelper.getNumbers(1, 3), args[1]);
        }
        return new ArrayList<>();
    }

    /**
     * Initializes the lists of tab complete values
     */
    private void initializeTabCompleteLists() {
        paymentTypes = new ArrayList<>();
        paymentTypes.add("item");
        paymentTypes.add("eco");
    }

}
