package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.cleanString;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.fixName;

/**
 * Helper class for getting abstract book information
 */
public final class BookHelper {

    private BookHelper() {

    }

    /**
     * Increases the generation of the given book, if necessary
     *
     * @param bookItem <p>The book item to increase the generation of</p>
     */
    public static void increaseGeneration(ItemStack bookItem) {
        BookMeta bookMeta = (BookMeta) bookItem.getItemMeta();
        if (BooksWithoutBordersConfig.changeGenerationOnCopy() && bookMeta != null && bookMeta.hasGeneration()) {
            bookMeta.setGeneration(BookHelper.getNextGeneration(bookMeta.getGeneration()));
            bookItem.setItemMeta(bookMeta);
        }
    }

    /**
     * Gets the next generation of the given book
     *
     * <p>If an original book is given, this will yield a copy of the original. If a copy of original is given, this
     * will yield a copy of a copy. In all other cases, the generation will stay the same</p>
     *
     * @param currentGeneration <p>The current generation of the book</p>
     * @return <p>The next generation of the book</p>
     */
    public static BookMeta.Generation getNextGeneration(BookMeta.Generation currentGeneration) {
        return switch (currentGeneration) {
            case ORIGINAL -> BookMeta.Generation.COPY_OF_ORIGINAL;
            case COPY_OF_ORIGINAL -> BookMeta.Generation.COPY_OF_COPY;
            default -> currentGeneration;
        };
    }

    /**
     * Gets the file name of the given book
     *
     * @param book   <p>The book to get the file of</p>
     * @param player <p>The player trying to do something with the book</p>
     * @return <p>The book file</p>
     */
    public static String getBookFile(BookMeta book, Player player) {
        String titleAuthorSeparator = BooksWithoutBordersConfig.getTitleAuthorSeparator();
        String bookName;
        if (!book.hasTitle()) {
            bookName = book.getTitle();
        } else {
            bookName = "Untitled";
        }

        String authorName;
        if (book.hasAuthor()) {
            authorName = book.getAuthor();
        } else {
            authorName = player.getName();
        }

        return fixName(cleanString(bookName + titleAuthorSeparator + authorName), false);
    }

    /**
     * Checks whether the given player is the author of a given book
     *
     * @param player <p>The player to check</p>
     * @param book   <p>The book to check</p>
     * @return <p>True if the player is not the book's author</p>
     */
    public static boolean isNotAuthor(Player player, BookMeta book) {
        String author = book.getAuthor();
        String playerName = InputCleaningHelper.cleanString(player.getName());
        if (author != null && playerName.equalsIgnoreCase(InputCleaningHelper.cleanString(author))) {
            return false;
        }

        BooksWithoutBorders.sendErrorMessage(player, "You must be the author of this book to use this command!");
        return true;
    }

}
