package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import org.bukkit.ChatColor;
import org.bukkit.command.CommandSender;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.regex.Pattern;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.cleanString;

/**
 * Helper class for dealing with files
 */
public final class FileHelper {

    private FileHelper() {
    }

    /**
     * Check whether an integer points to a public book
     *
     * @param possibleIndex <p>The string which might be a book index</p>
     * @return <p>True if the number is a book index</p>
     */
    public static boolean isBookListIndex(String possibleIndex) {
        File bookDirectory = new File(getBookFolder().replaceAll("[\\\\/]$", ""));

        try {
            //Tests if a load list number has been supplied
            int parsedLine = Integer.parseInt(possibleIndex);
            File[] foundFiles = bookDirectory.listFiles(File::isFile);

            return parsedLine >= 0 && parsedLine <= Objects.requireNonNull(foundFiles).length;
        } catch (NumberFormatException ignored) {
        }

        return false;
    }

    /**
     * Checks if a file path points to an actual book
     *
     * @param bookFile <p>The path to a book</p>
     * @return <p>True if the file exists and points to a book file</p>
     */
    public static boolean bookFileExists(String bookFile) {
        return ((new File(bookFile).isFile() && (bookFile.endsWith(".txt") ||
                bookFile.endsWith(".yml"))) || new File(bookFile + ".txt").isFile() ||
                new File(bookFile + ".yml").isFile()) && !bookFile.contains("../") && !bookFile.contains("..\\");
    }

    /**
     * Gets a book file given its path
     *
     * <p>This function adds extensions to the path as necessary, returning the first match.</p>
     *
     * @param bookPath <p>The path of the book to get</p>
     * @return <p>The file the path points to, or null otherwise</p>
     */
    public static File getBookFile(String bookPath) {
        if (!bookFileExists(bookPath)) {
            return null;
        }

        File bookFile = new File(bookPath);
        if (bookFile.exists()) {
            return bookFile;
        }

        File bookFileYml = new File(bookPath + ".yml");
        if (bookFileYml.exists()) {
            return bookFileYml;
        }

        File bookFileTxt = new File(bookPath + ".txt");
        if (bookFileTxt.exists()) {
            return bookFileTxt;
        }

        return null;
    }

    /**
     * Lists available files
     *
     * @param sender     <p>The command sender looking for files</p>
     * @param listPublic <p>Whether to list public or personal files</p>
     * @return <p>A list of available files</p>
     */
    public static List<String> listFiles(CommandSender sender, Boolean listPublic) {
        File file;
        if (listPublic) {
            file = new File(getBookFolder());
        } else {
            file = new File(getBookFolder() + cleanString(sender.getName()) + getSlash());
        }
        return FileHelper.listFiles(sender, file);
    }

    /**
     * Prints the available books
     *
     * @param sender     <p>The sender to display the books to</p>
     * @param listPublic <p>Whether to display public books</p>
     */
    public static void printBooks(CommandSender sender, boolean listPublic) {
        List<String> availableBooks = BooksWithoutBorders.getAvailableBooks(sender, listPublic);
        FileHelper.printFiles(sender, availableBooks);
    }

    /**
     * Prints a list of files
     *
     * @param sender   <p>The command sender to show the list to</p>
     * @param fileList <p>The files to list</p>
     */
    public static void printFiles(CommandSender sender, List<String> fileList) {
        BooksWithoutBorders.sendSuccessMessage(sender, "Available Books:");
        if (fileList == null) {
            return;
        }
        int listSize = fileList.size();
        for (int fileIndex = 0; fileIndex < listSize; fileIndex++) {
            sender.sendMessage(ChatColor.GRAY + "[" + (fileIndex + 1) + "] " + fileList.get(fileIndex));
        }
    }

    /**
     * Lists available files
     *
     * @param sender          <p>The command sender looking for files</p>
     * @param searchDirectory <p>The directory to search for files</p>
     * @return <p>A list of available files</p>
     */
    private static List<String> listFiles(CommandSender sender, File searchDirectory) {
        List<String> fileList = new ArrayList<>();
        File[] existingFiles = searchDirectory.listFiles();

        if (!searchDirectory.exists() || existingFiles == null || existingFiles.length == 0) {
            BooksWithoutBorders.sendErrorMessage(sender, "No books have been saved!");
            return null;
        }

        for (File foundFile : existingFiles) {
            if (foundFile.isFile()) {
                fileList.add(foundFile.getName());
            }
        }

        return fileList;
    }

    /**
     * Finds any duplicate book files
     *
     * @param foundFiles <p>The files to check for duplicates</p>
     * @param fileName   <p>The name of the file which might already exist</p>
     * @return <p>The number of found duplicates</p>
     */
    public static int findDuplicates(File[] foundFiles, String fileName) {
        int foundDuplicates = 0;
        for (File foundFile : foundFiles) {
            if (foundFile.getName().matches("(\\([0-9]+\\))?" + Pattern.quote(fileName) + "(\\.yml|\\.txt)?")) {
                foundDuplicates++;
            }
        }
        return foundDuplicates;
    }

    /**
     * Gets a buffered reader given an input stream
     *
     * @param inputStream <p>The input stream to read</p>
     * @return <p>A buffered reader reading the input stream</p>
     */
    public static BufferedReader getBufferedReaderFromInputStream(InputStream inputStream) {
        InputStreamReader inputStreamReader = new InputStreamReader(inputStream, StandardCharsets.UTF_8);
        return new BufferedReader(inputStreamReader);
    }

}
